=== modified file 'NEWS'
--- NEWS	2006-07-11 15:05:36 +0000
+++ NEWS	2006-07-13 19:18:49 +0000
@@ -2,6 +2,11 @@
 
   IMPROVEMENTS:
 
+   * The revision specifier "revno:" is extended to accept the syntax
+     revno:N:branch. For example,
+     revno:42:http://bazaar-vcs.org/bzr/bzr.dev/ means revision 42 in
+     bzr.dev.  (Matthieu Moy)
+
    * No default ignore rules are applied by bzr - only the rules in .bzrignore
      are considered. This fixes bugs with default rules being enforced no
      matter what. The old list of ignore rules from bzr is available by
@@ -83,6 +88,19 @@
      Also updates things like 'http+pycurl://' if pycurl is not present.
      (John Arbash Meinel) (Malone #47821, #52204)
 
+   * New env variable BZR_PROGRESS_BAR, sets the default progress bar type.
+     Can be set to 'none' or 'dummy' to disable the progress bar, 'dots' or 
+     'tty' to create the respective type. (John Arbash Meinel, #42197, #51107)
+
+   * Improve the help text for 'bzr diff' to explain what various options do.
+     (John Arbash Meinel, #6391)
+
+   * 'bzr uncommit -r 10' now uncommits revisions 11.. rather than uncommitting
+     revision 10. This makes -r10 more in line with what other commands do.
+     'bzr uncommit' also now saves the pending merges of the revisions that
+     were removed. So it is safe to uncommit after a merge, fix something,
+     and commit again. (John Arbash Meinel, #32526, #31426)
+
   BUG FIXES:
 
     * Fix shadowed definition of TestLocationConfig that caused some 
@@ -116,6 +134,33 @@
     * Normalize unicode filenames to ensure cross-platform consistency.
       (John Arbash Meinel, #43689)
 
+    * The argument parser can now handle '-' as an argument. Currently
+      no code interprets it specially (it is mostly handled as a file named 
+      '-'). But plugins, and future operations can use it.
+      (John Arbash meinel, #50984)
+
+    * Bundles can properly read binary files with a plain '\r' in them.
+      (John Arbash Meinel, #51927)
+
+    * Tuning iter_entries() to be more efficient (John Arbash Meinel, #5444)
+
+    * Lots of win32 fixes (the test suite passes again).
+      (John Arbash Meinel, #50155)
+
+    * Handle openbsd returning None for sys.getfilesystemencoding() (#41183) 
+
+    * Support ftp APPE (append) to allow Knits to be used over ftp (#42592)
+
+    * Removals are only committed if they match the filespec (or if there is
+      no filespec).  (#46635, Aaron Bentley)
+
+    * smart-add recurses through all supplied directories 
+      (John Arbash Meinel, #52578)
+
+    * Make the bundle reader extra lines before and after the bundle text.
+      This allows you to parse an email with the bundle inline.
+      (John Arbash Meinel, #49182)
+
   INTERNALS:
 
     * Combine the ignore rules into a single regex rather than looping over

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2006-07-02 05:28:12 +0000
+++ bzrlib/add.py	2006-07-12 14:24:06 +0000
@@ -117,6 +117,12 @@
             self.base_path = base_path
         self.raw_path = path
 
+    def __cmp__(self, other):
+        return cmp(self.raw_path, other.raw_path)
+
+    def __hash__(self):
+        return hash(self.raw_path)
+
 
 def smart_add_tree(tree, file_list, recurse=True, action=None, save=True):
     """Add files to version, optionally recursing into directories.
@@ -160,7 +166,7 @@
         kind = bzrlib.osutils.file_kind(abspath)
         if kind == 'directory':
             # schedule the dir for scanning
-            user_dirs.add(rf.raw_path)
+            user_dirs.add(rf)
         else:
             if not InventoryEntry.versionable_kind(kind):
                 raise errors.BadFileKindError(filename=abspath, kind=kind)
@@ -182,11 +188,12 @@
     # only walk the minimal parents needed: we have user_dirs to override
     # ignores.
     prev_dir = None
+
+    is_inside = bzrlib.osutils.is_inside_or_parent_of_any
     for path in sorted(user_dirs):
-        if (prev_dir is None or not
-            bzrlib.osutils.is_inside_or_parent_of_any([prev_dir], path)):
-            dirs_to_add.append((rf, None))
-        prev_dir = path
+        if (prev_dir is None or not is_inside([prev_dir], path.raw_path)):
+            dirs_to_add.append((path, None))
+        prev_dir = path.raw_path
 
     # this will eventually be *just* directories, right now it starts off with 
     # just directories.

=== modified file 'bzrlib/benchmarks/__init__.py'
--- bzrlib/benchmarks/__init__.py	2006-07-06 02:27:26 +0000
+++ bzrlib/benchmarks/__init__.py	2006-07-11 14:47:00 +0000
@@ -57,7 +57,7 @@
         
         No files change are included.
         """
-        tree = BzrDir.create_standalone_workingtree(directory_name)
+        tree = bzrdir.BzrDir.create_standalone_workingtree(directory_name)
         tree.lock_write()
         tree.branch.lock_write()
         tree.branch.repository.lock_write()
@@ -82,7 +82,7 @@
         commit in its revision-history is a merge another commit in the other
         tree.
         """
-        tree = BzrDir.create_standalone_workingtree(directory_name)
+        tree = bzrdir.BzrDir.create_standalone_workingtree(directory_name)
         tree.lock_write()
         try:
             tree2 = tree.bzrdir.sprout('tree2').open_workingtree()

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-07-09 20:12:49 +0000
+++ bzrlib/builtins.py	2006-07-13 12:24:32 +0000
@@ -1078,7 +1078,7 @@
 
 
 class cmd_diff(Command):
-    """Show differences in working tree.
+    """Show differences in the working tree or between revisions.
     
     If files are listed, only the changes in those files are listed.
     Otherwise, all changes for the tree are listed.
@@ -1088,11 +1088,17 @@
 
     examples:
         bzr diff
+            Shows the difference in the working tree versus the last commit
         bzr diff -r1
+            Difference between the working tree and revision 1
         bzr diff -r1..2
+            Difference between revision 2 and revision 1
         bzr diff --diff-prefix old/:new/
+            Same as 'bzr diff' but prefix paths with old/ and new/
         bzr diff bzr.mine bzr.dev
+            Show the differences between the two working trees
         bzr diff foo.c
+            Show just the differences for 'foo.c'
     """
     # TODO: Option to use external diff command; could be GNU diff, wdiff,
     #       or a graphical diff.
@@ -1142,6 +1148,10 @@
                 # FIXME diff those two files. rbc 20051123
                 raise BzrCommandError("Files are in different branches")
             file_list = None
+        except NotBranchError:
+            # Don't raise an error when bzr diff is called from
+            # outside a working tree.
+            tree1, tree2 = None, None
         if revision is not None:
             if tree2 is not None:
                 raise BzrCommandError("Can't specify -r with two branches")
@@ -1820,6 +1830,13 @@
                 self.outf.write(c.username() + '\n')
             return
 
+        # display a warning if an email address isn't included in the given name.
+        try:
+            config.extract_email_address(name)
+        except BzrError, e:
+            warning('"%s" does not seem to contain an email address.  '
+                    'This is allowed, but not recommended.', name)
+        
         # use global config unless --branch given
         if branch:
             c = Branch.open_containing('.')[0].get_config()
@@ -2620,10 +2637,10 @@
     takes_args = ['location?']
     aliases = []
 
-    def run(self, location=None, 
+    def run(self, location=None,
             dry_run=False, verbose=False,
             revision=None, force=False):
-        from bzrlib.log import log_formatter
+        from bzrlib.log import log_formatter, show_log
         import sys
         from bzrlib.uncommit import uncommit
 
@@ -2637,18 +2654,33 @@
             tree = None
             b = control.open_branch()
 
+        rev_id = None
         if revision is None:
             revno = b.revno()
-            rev_id = b.last_revision()
         else:
-            revno, rev_id = revision[0].in_history(b)
+            # 'bzr uncommit -r 10' actually means uncommit
+            # so that the final tree is at revno 10.
+            # but bzrlib.uncommit.uncommit() actually uncommits
+            # the revisions that are supplied.
+            # So we need to offset it by one
+            revno = revision[0].in_history(b).revno+1
+
+        if revno <= b.revno():
+            rev_id = b.get_rev_id(revno)
         if rev_id is None:
-            print 'No revisions to uncommit.'
-
-        for r in range(revno, b.revno()+1):
-            rev_id = b.get_rev_id(r)
-            lf = log_formatter('short', to_file=sys.stdout,show_timezone='original')
-            lf.show(r, b.repository.get_revision(rev_id), None)
+            self.outf.write('No revisions to uncommit.\n')
+            return 1
+
+        lf = log_formatter('short',
+                           to_file=self.outf,
+                           show_timezone='original')
+
+        show_log(b,
+                 lf,
+                 verbose=False,
+                 direction='forward',
+                 start_revision=revno,
+                 end_revision=b.revno())
 
         if dry_run:
             print 'Dry-run, pretending to remove the above revisions.'

=== modified file 'bzrlib/bundle/serializer/__init__.py'
--- bzrlib/bundle/serializer/__init__.py	2006-07-10 23:22:41 +0000
+++ bzrlib/bundle/serializer/__init__.py	2006-07-11 16:26:34 +0000
@@ -31,7 +31,7 @@
 CHANGESET_OLD_HEADER_RE = re.compile(r'^# Bazaar-NG changeset v(?P<version>\d+[\w.]*)\n$')
 
 
-_serializers = {} 
+_serializers = {}
 
 
 def _get_filename(f):

=== modified file 'bzrlib/bundle/serializer/v08.py'
--- bzrlib/bundle/serializer/v08.py	2006-06-22 20:01:47 +0000
+++ bzrlib/bundle/serializer/v08.py	2006-07-11 16:26:34 +0000
@@ -19,8 +19,9 @@
 
 import os
 
-from bzrlib.bundle.serializer import (BundleSerializer, 
-                                      BUNDLE_HEADER, 
+from bzrlib import errors
+from bzrlib.bundle.serializer import (BundleSerializer,
+                                      BUNDLE_HEADER,
                                       format_highres_date,
                                       unpack_highres_date,
                                      )
@@ -28,7 +29,6 @@
 from bzrlib.bundle.bundle_data import (RevisionInfo, BundleInfo, BundleTree)
 from bzrlib.delta import compare_trees
 from bzrlib.diff import internal_diff
-import bzrlib.errors as errors
 from bzrlib.osutils import pathjoin
 from bzrlib.progress import DummyProgress
 from bzrlib.revision import NULL_REVISION
@@ -367,6 +367,8 @@
             # which does not start with '#'
             if line is None or line == '\n':
                 break
+            if not line.startswith('#'):
+                continue
             found_something = True
             self._handle_next(line)
         if not found_something:
@@ -378,7 +380,7 @@
         """Read in a key-value pair
         """
         if not line.startswith('#'):
-            raise MalformedHeader('Bzr header did not start with #')
+            raise errors.MalformedHeader('Bzr header did not start with #')
         line = line[1:-1].decode('utf-8') # Remove the '#' and '\n'
         if line[:indent] == ' '*indent:
             line = line[indent:]
@@ -395,7 +397,7 @@
             key = line[:-1]
             value = self._read_many(indent=indent+2)
         else:
-            raise MalformedHeader('While looking for key: value pairs,'
+            raise errors.MalformedHeader('While looking for key: value pairs,'
                     ' did not find the colon %r' % (line))
 
         key = key.replace(' ', '_')
@@ -415,10 +417,10 @@
             if getattr(revision_info, key) is None:
                 setattr(revision_info, key, value)
             else:
-                raise MalformedHeader('Duplicated Key: %s' % key)
+                raise errors.MalformedHeader('Duplicated Key: %s' % key)
         else:
             # What do we do with a key we don't recognize
-            raise MalformedHeader('Unknown Key: "%s"' % key)
+            raise errors.MalformedHeader('Unknown Key: "%s"' % key)
     
     def _read_many(self, indent):
         """If a line ends with no entry, that means that it should be
@@ -455,7 +457,7 @@
         for line in self._next():
             if first:
                 if not line.startswith('==='):
-                    raise MalformedPatches('The first line of all patches'
+                    raise errors.MalformedPatches('The first line of all patches'
                         ' should be a bzr meta line "==="'
                         ': %r' % line)
                 action = line[4:-1].decode('utf-8')
@@ -493,8 +495,9 @@
         """
         for line in self._next():
             self._handle_next(line)
+            if self._next_line is None:
+                break
             if not self._next_line.startswith('#'):
+                # Consume the trailing \n and stop processing
                 self._next().next()
                 break
-            if self._next_line is None:
-                break

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-07-14 05:02:37 +0000
+++ bzrlib/commands.py	2006-07-15 14:29:03 +0000
@@ -368,6 +368,9 @@
             if argsover:
                 args.append(a)
                 continue
+            elif a == '-':
+                args.append(a)
+                continue
             elif a == '--':
                 # We've received a standalone -- No more flags
                 argsover = True

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-06-21 21:35:06 +0000
+++ bzrlib/commit.py	2006-07-11 15:04:57 +0000
@@ -497,6 +497,10 @@
         None; inventory entries that are carried over untouched have their
         revision set to their prior value.
         """
+        # ESEPARATIONOFCONCERNS: this function is diffing and using the diff
+        # results to create a new inventory at the same time, which results
+        # in bugs like #46635.  Any reason not to use/enhance compare_trees?
+        # ADHB 11-07-2006
         mutter("Selecting files for commit with filter %s", self.specific_files)
         # iter_entries does not visit the ROOT_ID node so we need to call
         # self._emit_progress_update once by hand.
@@ -534,6 +538,20 @@
             else:
                 self.reporter.snapshot_change(change, path)
 
+        if not self.specific_files:
+            return
+
+        # ignore removals that don't match filespec
+        for path, new_ie in self.basis_inv.iter_entries():
+            if new_ie.file_id in self.work_inv:
+                continue
+            if is_inside_any(self.specific_files, path):
+                continue
+            ie = new_ie.copy()
+            ie.revision = None
+            self.builder.record_entry_contents(ie, self.parent_invs, path,
+                                               self.basis_tree)
+
     def _emit_progress_update(self):
         """Emit an update to the progress bar."""
         self.pb.update("Committing", self.pb_count, self.pb_total)

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-07-05 05:18:06 +0000
+++ bzrlib/diff.py	2006-07-12 12:36:57 +0000
@@ -256,10 +256,14 @@
     The more general form is show_diff_trees(), where the caller
     supplies any two trees.
     """
-    output = sys.stdout
     def spec_tree(spec):
-        revision_id = spec.in_store(tree.branch).rev_id
-        return tree.branch.repository.revision_tree(revision_id)
+        if tree:
+            revision = spec.in_store(tree.branch)
+        else:
+            revision = spec.in_store(None)
+        revision_id = revision.rev_id
+        branch = revision.branch
+        return branch.repository.revision_tree(revision_id)
     if old_revision_spec is None:
         old_tree = tree.basis_tree()
     else:

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-07-11 15:05:36 +0000
+++ bzrlib/errors.py	2006-07-12 02:20:47 +0000
@@ -970,6 +970,14 @@
     """A nested progress bar was not 'finished' correctly."""
 
 
+class InvalidProgressBarType(BzrNewError):
+    """Environment variable BZR_PROGRESS_BAR='%(bar_type)s is not a supported type
+Select one of: %(valid_types)s"""
+
+    def __init__(self, bar_type, valid_types):
+        BzrNewError.__init__(self, bar_type=bar_type, valid_types=valid_types)
+
+
 class UnsupportedOperation(BzrNewError):
     """The method %(mname)s is not supported on objects of type %(tname)s."""
     def __init__(self, method, method_self):

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2006-03-20 18:34:33 +0000
+++ bzrlib/hashcache.py	2006-07-12 05:22:48 +0000
@@ -42,20 +42,6 @@
 FP_CTIME_COLUMN = 2
 FP_MODE_COLUMN = 5
 
-def _fingerprint(abspath):
-    try:
-        fs = os.lstat(abspath)
-    except OSError:
-        # might be missing, etc
-        return None
-
-    if stat.S_ISDIR(fs.st_mode):
-        return None
-
-    # we discard any high precision because it's not reliable; perhaps we
-    # could do better on some systems?
-    return (fs.st_size, long(fs.st_mtime),
-            long(fs.st_ctime), fs.st_ino, fs.st_dev, fs.st_mode)
 
 
 class HashCache(object):
@@ -131,7 +117,7 @@
         
         for inum, path, cache_entry in prep:
             abspath = pathjoin(self.root, path)
-            fp = _fingerprint(abspath)
+            fp = self._fingerprint(abspath)
             self.stat_count += 1
             
             cache_fp = cache_entry[1]
@@ -142,13 +128,12 @@
                 self.needs_write = True
                 del self._cache[path]
 
-
     def get_sha1(self, path):
         """Return the sha1 of a file.
         """
         abspath = pathjoin(self.root, path)
         self.stat_count += 1
-        file_fp = _fingerprint(abspath)
+        file_fp = self._fingerprint(abspath)
         
         if not file_fp:
             # not a regular file or not existing
@@ -164,22 +149,26 @@
             cache_sha1, cache_fp = None, None
 
         if cache_fp == file_fp:
+            ## mutter("hashcache hit for %s %r -> %s", path, file_fp, cache_sha1)
+            ## mutter("now = %s", time.time())
             self.hit_count += 1
             return cache_sha1
         
         self.miss_count += 1
 
-
         mode = file_fp[FP_MODE_COLUMN]
         if stat.S_ISREG(mode):
-            digest = sha_file(file(abspath, 'rb', buffering=65000))
+            digest = self._really_sha1_file(abspath)
         elif stat.S_ISLNK(mode):
             digest = sha.new(os.readlink(abspath)).hexdigest()
         else:
             raise BzrError("file %r: unknown file stat mode: %o"%(abspath,mode))
 
-        now = int(time.time())
-        if file_fp[FP_MTIME_COLUMN] >= now or file_fp[FP_CTIME_COLUMN] >= now:
+        # window of 3 seconds to allow for 2s resolution on windows,
+        # unsynchronized file servers, etc.
+        cutoff = self._cutoff_time()
+        if file_fp[FP_MTIME_COLUMN] >= cutoff \
+                or file_fp[FP_CTIME_COLUMN] >= cutoff:
             # changed too recently; can't be cached.  we can
             # return the result and it could possibly be cached
             # next time.
@@ -191,16 +180,24 @@
             # need to let sufficient time elapse before we may cache this entry
             # again.  If we didn't do this, then, for example, a very quick 1
             # byte replacement in the file might go undetected.
-            self.danger_count += 1 
+            ## mutter('%r modified too recently; not caching', path)
+            self.danger_count += 1
             if cache_fp:
                 self.removed_count += 1
                 self.needs_write = True
                 del self._cache[path]
         else:
+            ## mutter('%r added to cache: now=%f, mtime=%d, ctime=%d',
+            ##        path, time.time(), file_fp[FP_MTIME_COLUMN],
+            ##        file_fp[FP_CTIME_COLUMN])
             self.update_count += 1
             self.needs_write = True
             self._cache[path] = (digest, file_fp)
         return digest
+
+    def _really_sha1_file(self, abspath):
+        """Calculate the SHA1 of a file by reading the full text"""
+        return sha_file(file(abspath, 'rb', buffering=65000))
         
     def write(self):
         """Write contents of cache to file."""
@@ -218,10 +215,10 @@
                 print >>outf
             outf.commit()
             self.needs_write = False
-            mutter("write hash cache: %s hits=%d misses=%d stat=%d recent=%d updates=%d",
-                   self.cache_file_name(), self.hit_count, self.miss_count,
-                   self.stat_count,
-                   self.danger_count, self.update_count)
+            ## mutter("write hash cache: %s hits=%d misses=%d stat=%d recent=%d updates=%d",
+            ##        self.cache_file_name(), self.hit_count, self.miss_count,
+            ##        self.stat_count,
+            ##        self.danger_count, self.update_count)
         finally:
             if not outf.closed:
                 outf.abort()
@@ -244,7 +241,6 @@
             self.needs_write = True
             return
 
-
         hdr = inf.readline()
         if hdr != CACHE_HEADER:
             mutter('cache header marker not found at top of %s;'
@@ -275,7 +271,24 @@
             self._cache[path] = (sha1, fp)
 
         self.needs_write = False
+
+    def _cutoff_time(self):
+        """Return cutoff time.
+
+        Files modified more recently than this time are at risk of being
+        undetectably modified and so can't be cached.
+        """
+        return int(time.time()) - 3
            
-
-
-        
+    def _fingerprint(self, abspath):
+        try:
+            fs = os.lstat(abspath)
+        except OSError:
+            # might be missing, etc
+            return None
+        if stat.S_ISDIR(fs.st_mode):
+            return None
+        # we discard any high precision because it's not reliable; perhaps we
+        # could do better on some systems?
+        return (fs.st_size, long(fs.st_mtime),
+                long(fs.st_ctime), fs.st_ino, fs.st_dev, fs.st_mode)

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2006-06-21 02:33:40 +0000
+++ bzrlib/progress.py	2006-07-10 23:00:44 +0000
@@ -39,17 +39,16 @@
 import sys
 import time
 import os
-from collections import deque
-
 
 import bzrlib.errors as errors
-from bzrlib.trace import mutter 
+from bzrlib.trace import mutter
 
 
 def _supports_progress(f):
-    if not hasattr(f, 'isatty'):
+    isatty = getattr(f, 'isatty', None)
+    if isatty is None:
         return False
-    if not f.isatty():
+    if not isatty():
         return False
     if os.environ.get('TERM') == 'dumb':
         # e.g. emacs compile window
@@ -57,16 +56,31 @@
     return True
 
 
+_progress_bar_types = {}
+
 
 def ProgressBar(to_file=None, **kwargs):
     """Abstract factory"""
     if to_file is None:
         to_file = sys.stderr
-    if _supports_progress(to_file):
-        return TTYProgressBar(to_file=to_file, **kwargs)
+    requested_bar_type = os.environ.get('BZR_PROGRESS_BAR')
+    # An value of '' or not set reverts to standard processing
+    if requested_bar_type in (None, ''):
+        if _supports_progress(to_file):
+            return TTYProgressBar(to_file=to_file, **kwargs)
+        else:
+            return DotsProgressBar(to_file=to_file, **kwargs)
     else:
-        return DotsProgressBar(to_file=to_file, **kwargs)
-    
+        # Minor sanitation to prevent spurious errors
+        requested_bar_type = requested_bar_type.lower().strip()
+        # TODO: jam 20060710 Arguably we shouldn't raise an exception
+        #       but should instead just disable progress bars if we
+        #       don't recognize the type
+        if requested_bar_type not in _progress_bar_types:
+            raise errors.InvalidProgressBarType(requested_bar_type,
+                                                _progress_bar_types.keys())
+        return _progress_bar_types[requested_bar_type](to_file=to_file, **kwargs)
+
  
 class ProgressBarStack(object):
     """A stack of progress bars."""
@@ -93,7 +107,7 @@
         self._show_count = show_count
         self._to_messages_file = to_messages_file
         self._stack = []
-        self._klass = klass or TTYProgressBar
+        self._klass = klass or ProgressBar
 
     def top(self):
         if len(self._stack) != 0:
@@ -206,6 +220,10 @@
         return DummyProgress(**kwargs)
 
 
+_progress_bar_types['dummy'] = DummyProgress
+_progress_bar_types['none'] = DummyProgress
+
+
 class DotsProgressBar(_BaseProgressBar):
 
     def __init__(self, **kwargs):
@@ -233,6 +251,9 @@
     def child_update(self, message, current, total):
         self.tick()
 
+
+_progress_bar_types['dots'] = DotsProgressBar
+
     
 class TTYProgressBar(_BaseProgressBar):
     """Progress bar display object.
@@ -262,8 +283,10 @@
         _BaseProgressBar.__init__(self, **kwargs)
         self.spin_pos = 0
         self.width = terminal_width()
-        self.last_updates = deque()
+        self.last_updates = []
+        self._max_last_updates = 10
         self.child_fraction = 0
+        self._have_output = False
     
 
     def throttle(self, old_msg):
@@ -284,6 +307,8 @@
             return True
 
         self.last_updates.append(now - self.last_update)
+        # Don't let the queue grow without bound
+        self.last_updates = self.last_updates[-self._max_last_updates:]
         self.last_update = now
         return False
         
@@ -405,13 +430,19 @@
 
         assert len(m) < self.width
         self.to_file.write('\r' + m.ljust(self.width - 1))
+        self._have_output = True
         #self.to_file.flush()
             
     def clear(self):        
-        self.to_file.write('\r%s\r' % (' ' * (self.width - 1)))
+        if self._have_output:
+            self.to_file.write('\r%s\r' % (' ' * (self.width - 1)))
+        self._have_output = False
         #self.to_file.flush()        
 
 
+_progress_bar_types['tty'] = TTYProgressBar
+
+
 class ChildProgress(_BaseProgressBar):
     """A progress indicator that pushes its data to the parent"""
 
@@ -487,8 +518,6 @@
     assert total_duration >= elapsed
 
     if last_updates and len(last_updates) >= n_recent:
-        while len(last_updates) > n_recent:
-            last_updates.popleft()
         avg = sum(last_updates) / float(len(last_updates))
         time_left = avg * (total - current)
 

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2006-07-04 17:23:00 +0000
+++ bzrlib/revision.py	2006-07-08 19:32:30 +0000
@@ -75,8 +75,7 @@
         return not self.__eq__(other)
 
     def _check_properties(self):
-        """Verify that all revision properties are OK.
-        """
+        """Verify that all revision properties are OK."""
         for name, value in self.properties.iteritems():
             if not isinstance(name, basestring) or contains_whitespace(name):
                 raise ValueError("invalid property name %r" % name)

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2006-05-24 00:06:33 +0000
+++ bzrlib/revisionspec.py	2006-07-08 19:32:30 +0000
@@ -149,7 +149,10 @@
             raise NoSuchRevision(branch, str(self.spec))
 
     def in_history(self, branch):
-        revs = branch.revision_history()
+        if branch:
+            revs = branch.revision_history()
+        else:
+            revs = None
         return self._match_on_and_check(branch, revs)
 
         # FIXME: in_history is somewhat broken,
@@ -190,10 +193,21 @@
 
     def _match_on(self, branch, revs):
         """Lookup a revision by revision number"""
-        try:
-            return RevisionInfo(branch, int(self.spec))
-        except ValueError:
-            return RevisionInfo(branch, None)
+        if self.spec.find(':') == -1:
+            try:
+                return RevisionInfo(branch, int(self.spec))
+            except ValueError:
+                return RevisionInfo(branch, None)
+        else:
+            from branch import Branch
+            revname = self.spec[self.spec.find(':')+1:]
+            other_branch = Branch.open_containing(revname)[0]
+            try:
+                revno = int(self.spec[:self.spec.find(':')])
+            except ValueError:
+                return RevisionInfo(other_branch, None)
+            revid = other_branch.get_rev_id(revno)
+            return RevisionInfo(other_branch, revno)
 
 SPEC_TYPES.append(RevisionSpec_revno)
 

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2006-06-20 05:32:16 +0000
+++ bzrlib/status.py	2006-07-13 00:25:34 +0000
@@ -165,8 +165,10 @@
     for merge in new.pending_merges():
         ignore.add(merge)
         try:
+            from bzrlib.osutils import terminal_width
+            width = terminal_width()
             m_revision = branch.repository.get_revision(merge)
-            print >> to_file, ' ', line_log(m_revision, 77)
+            print >> to_file, ' ', line_log(m_revision, width - 3)
             inner_merges = branch.repository.get_ancestry(merge)
             assert inner_merges[0] == None
             inner_merges.pop(0)
@@ -175,7 +177,7 @@
                 if mmerge in ignore:
                     continue
                 mm_revision = branch.repository.get_revision(mmerge)
-                print >> to_file, '   ', line_log(mm_revision, 75)
+                print >> to_file, '   ', line_log(mm_revision, width - 5)
                 ignore.add(mmerge)
         except errors.NoSuchRevision:
             print >> to_file, ' ', merge

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2006-06-23 16:26:26 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2006-07-11 15:04:57 +0000
@@ -287,3 +287,16 @@
         # version or the u2 version.
         self.build_tree_contents([('u1/hosts', 'merge resolution\n')])
         self.run_bzr('commit', '-m', 'checkin merge of the offline work from u1', 'u1')
+
+    def test_commit_respects_spec_for_removals(self):
+        """Commit with a file spec should only commit removals that match"""
+        t = self.make_branch_and_tree('.')
+        self.build_tree(['file-a', 'dir-a/', 'dir-a/file-b'])
+        t.add(['file-a', 'dir-a', 'dir-a/file-b'])
+        t.commit('Create')
+        t.remove(['file-a', 'dir-a/file-b'])
+        os.chdir('dir-a')
+        result = self.run_bzr('commit', '.', '-m' 'removed file-b')[1]
+        self.assertNotContainsRe(result, 'file-a')
+        result = self.run_bzr('status')[0]
+        self.assertContainsRe(result, 'removed:\n  file-a')

=== modified file 'bzrlib/tests/blackbox/test_diff.py'
--- bzrlib/tests/blackbox/test_diff.py	2006-07-05 05:03:34 +0000
+++ bzrlib/tests/blackbox/test_diff.py	2006-07-08 20:38:36 +0000
@@ -157,6 +157,26 @@
                               "+contents of branch1/file\n"
                               "\n", subst_dates(out))
 
+    def test_diff_revno_branches(self):
+        self.example_branches()
+        print >> open('branch2/file', 'wb'), 'even newer content'
+        self.run_bzr_captured(['commit', '-m', 'update file once more', 'branch2'])
+
+        out, err = self.run_bzr_captured(['diff', '-r', 'revno:1:branch2..revno:1:branch1'],
+                                         retcode=0)
+        self.assertEquals('', err)
+        self.assertEquals('', out)
+        out, ett = self.run_bzr_captured(['diff', '-r', 'revno:2:branch2..revno:1:branch1'],
+                                         retcode=1)
+        self.assertEquals('', err)
+        self.assertEqualDiff("=== modified file 'file'\n"
+                              "--- file\tYYYY-MM-DD HH:MM:SS +ZZZZ\n"
+                              "+++ file\tYYYY-MM-DD HH:MM:SS +ZZZZ\n"
+                              "@@ -1,1 +1,1 @@\n"
+                              "-new content\n"
+                              "+contents of branch1/file\n"
+                              "\n", subst_dates(out))
+
     def example_branch2(self):
         self.build_tree(['branch1/', 'branch1/file1'], line_endings='binary')
         self.capture('init branch1')

=== modified file 'bzrlib/tests/blackbox/test_uncommit.py'
--- bzrlib/tests/blackbox/test_uncommit.py	2006-04-02 02:38:13 +0000
+++ bzrlib/tests/blackbox/test_uncommit.py	2006-07-11 21:09:40 +0000
@@ -1,68 +1,89 @@
-"""\
-Test the uncommit command.
-"""
+# Copyright (C) 2005, 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test the uncommit command."""
 
 import os
 
+from bzrlib import uncommit, workingtree
 from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.errors import BzrError, BoundBranchOutOfDate
-from bzrlib.uncommit import uncommit
-from bzrlib.tests import TestCaseInTempDir
-
-class TestUncommit(TestCaseInTempDir):
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestUncommit(TestCaseWithTransport):
+    def create_simple_tree(self):
+        wt = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b', 'tree/c'])
+        wt.add(['a', 'b', 'c'])
+        wt.commit('initial commit', rev_id='a1')
+
+        open('tree/a', 'wb').write('new contents of a\n')
+        wt.commit('second commit', rev_id='a2')
+
+        return wt
+
     def test_uncommit(self):
         """Test uncommit functionality."""
-        bzr = self.capture 
-        os.mkdir('branch')
-        os.chdir('branch')
-        bzr('init')
-        self.build_tree(['a', 'b', 'c'])
-
-        bzr('add')
-        bzr('commit -m initial')
-
-        self.assertEquals(bzr('revno'), '1\n')
-
-        open('a', 'wb').write('new contents of a\n')
-        self.assertEquals(bzr('status'), 'modified:\n  a\n')
-        bzr('commit -m second')
-
-        self.assertEquals(bzr('status'), '')
-        self.assertEquals(bzr('revno'), '2\n')
-
-        txt = bzr('uncommit --dry-run --force')
-        self.failIfEqual(txt.find('Dry-run'), -1)
-
-        self.assertEquals(bzr('status'), '')
-        self.assertEquals(bzr('revno'), '2\n')
-
-        txt = bzr('uncommit --force')
-
-        self.assertEquals(bzr('revno'), '1\n')
-        self.assertEquals(bzr('status'), 'modified:\n  a\n')
-        
-        bzr('checkout . ../checkout')
-        os.chdir('../checkout')
-        self.assertEquals("", bzr('status'))
-        self.assertEquals(bzr('revno'), '1\n')
-
-        open('a', 'wb').write('new contents of a\n')
-        self.assertEquals(bzr('status'), 'modified:\n  a\n')
-        bzr('commit -m second')
-
-        self.assertEquals(bzr('status'), '')
-        self.assertEquals(bzr('revno'), '2\n')
-
-        txt = bzr('uncommit --dry-run --force')
-        self.failIfEqual(txt.find('Dry-run'), -1)
-
-        self.assertEquals(bzr('status'), '')
-        self.assertEquals(bzr('revno'), '2\n')
-
-        txt = bzr('uncommit --force')
-
-        self.assertEquals(bzr('revno'), '1\n')
-        self.assertEquals(bzr('status'), 'modified:\n  a\n')
+        wt = self.create_simple_tree()
+
+        os.chdir('tree')
+        out, err = self.run_bzr('uncommit', '--dry-run', '--force')
+        self.assertContainsRe(out, 'Dry-run')
+        self.assertNotContainsRe(out, 'initial commit')
+        self.assertContainsRe(out, 'second commit')
+
+        # Nothing has changed
+        self.assertEqual('a2', wt.last_revision())
+
+        # Uncommit, don't prompt
+        out, err = self.run_bzr('uncommit', '--force')
+        self.assertNotContainsRe(out, 'initial commit')
+        self.assertContainsRe(out, 'second commit')
+
+        # This should look like we are back in revno 1
+        self.assertEqual('a1', wt.last_revision())
+        out, err = self.run_bzr('status')
+        self.assertEquals(out, 'modified:\n  a\n')
+
+    def test_uncommit_checkout(self):
+        wt = self.create_simple_tree()
+
+        checkout_tree = wt.bzrdir.sprout('checkout').open_workingtree()
+        checkout_tree.branch.bind(wt.branch)
+
+        self.assertEqual('a2', checkout_tree.last_revision())
+
+        os.chdir('checkout')
+        out, err = self.run_bzr('uncommit', '--dry-run', '--force')
+        self.assertContainsRe(out, 'Dry-run')
+        self.assertNotContainsRe(out, 'initial commit')
+        self.assertContainsRe(out, 'second commit')
+
+        self.assertEqual('a2', checkout_tree.last_revision())
+
+        out, err = self.run_bzr('uncommit', '--force')
+        self.assertNotContainsRe(out, 'initial commit')
+        self.assertContainsRe(out, 'second commit')
+
+        # uncommit in a checkout should uncommit the parent branch
+        # (but doesn't effect the other working tree)
+        self.assertEquals('a1', checkout_tree.last_revision())
+        self.assertEquals('a1', wt.branch.last_revision())
+        self.assertEquals('a2', wt.last_revision())
 
     def test_uncommit_bound(self):
         os.mkdir('a')
@@ -75,12 +96,103 @@
         t.commit('commit 3')
         b = t.bzrdir.sprout('b').open_branch()
         b.bind(t.branch)
-        uncommit(b)
+        uncommit.uncommit(b)
         t.set_last_revision(t.branch.last_revision())
         self.assertEqual(len(b.revision_history()), 2)
         self.assertEqual(len(t.branch.revision_history()), 2)
         t.commit('commit 3b')
-        self.assertRaises(BoundBranchOutOfDate, uncommit, b)
+        self.assertRaises(BoundBranchOutOfDate, uncommit.uncommit, b)
         b.pull(t.branch)
-        uncommit(b)
-
+        uncommit.uncommit(b)
+
+    def test_uncommit_revision(self):
+        wt = self.create_simple_tree()
+
+        os.chdir('tree')
+        out, err = self.run_bzr('uncommit', '-r1', '--force')
+
+        self.assertNotContainsRe(out, 'initial commit')
+        self.assertContainsRe(out, 'second commit')
+        self.assertEqual('a1', wt.last_revision())
+        self.assertEqual('a1', wt.branch.last_revision())
+
+    def test_uncommit_neg_1(self):
+        wt = self.create_simple_tree()
+        os.chdir('tree')
+        out, err = self.run_bzr('uncommit', '-r', '-1', retcode=1)
+        self.assertEqual('No revisions to uncommit.\n', out)
+
+    def test_uncommit_merges(self):
+        wt = self.create_simple_tree()
+
+        tree2 = wt.bzrdir.sprout('tree2').open_workingtree()
+
+        tree2.commit('unchanged', rev_id='b3')
+        tree2.commit('unchanged', rev_id='b4')
+
+        wt.branch.fetch(tree2.branch)
+        wt.set_pending_merges(['b4'])
+        wt.commit('merge b4', rev_id='a3')
+
+        self.assertEqual('a3', wt.last_revision())
+        self.assertEqual([], wt.pending_merges())
+
+        os.chdir('tree')
+        out, err = self.run_bzr('uncommit', '--force')
+
+        self.assertEqual('a2', wt.last_revision())
+        self.assertEqual(['b4'], wt.pending_merges())
+
+    def test_uncommit_multiple_merge(self):
+        wt = self.create_simple_tree()
+
+        tree2 = wt.bzrdir.sprout('tree2').open_workingtree()
+
+        tree2.commit('unchanged', rev_id='b3')
+        wt.branch.fetch(tree2.branch)
+        wt.set_pending_merges(['b3'])
+        wt.commit('merge b3', rev_id='a3')
+
+        tree2.commit('unchanged', rev_id='b4')
+        wt.branch.fetch(tree2.branch)
+        wt.set_pending_merges(['b4'])
+        wt.commit('merge b4', rev_id='a4')
+
+        self.assertEqual('a4', wt.last_revision())
+        self.assertEqual([], wt.pending_merges())
+
+        os.chdir('tree')
+        out, err = self.run_bzr('uncommit', '--force', '-r', '2')
+
+        self.assertEqual('a2', wt.last_revision())
+        self.assertEqual(['b3', 'b4'], wt.pending_merges())
+
+    def test_uncommit_octopus_merge(self):
+        # Check that uncommit keeps the pending merges in the same order
+        wt = self.create_simple_tree()
+
+        tree2 = wt.bzrdir.sprout('tree2').open_workingtree()
+        tree3 = wt.bzrdir.sprout('tree3').open_workingtree()
+
+        tree2.commit('unchanged', rev_id='b3')
+        tree3.commit('unchanged', rev_id='c3')
+        wt.branch.fetch(tree2.branch)
+        wt.branch.fetch(tree3.branch)
+        wt.set_pending_merges(['b3', 'c3'])
+        wt.commit('merge b3, c3', rev_id='a3')
+
+        tree2.commit('unchanged', rev_id='b4')
+        tree3.commit('unchanged', rev_id='c4')
+        wt.branch.fetch(tree2.branch)
+        wt.branch.fetch(tree3.branch)
+        wt.set_pending_merges(['c4', 'b4'])
+        wt.commit('merge b4, c4', rev_id='a4')
+
+        self.assertEqual('a4', wt.last_revision())
+        self.assertEqual([], wt.pending_merges())
+
+        os.chdir('tree')
+        out, err = self.run_bzr('uncommit', '--force', '-r', '2')
+
+        self.assertEqual('a2', wt.last_revision())
+        self.assertEqual(['b3', 'c3', 'c4', 'b4'], wt.pending_merges())

=== modified file 'bzrlib/tests/blackbox/test_whoami.py'
--- bzrlib/tests/blackbox/test_whoami.py	2006-07-03 08:52:44 +0000
+++ bzrlib/tests/blackbox/test_whoami.py	2006-07-12 16:29:39 +0000
@@ -40,7 +40,8 @@
         """branch specific user identity works."""
         wt = self.make_branch_and_tree('.')
         b = bzrlib.branch.Branch.open('.')
-        b.get_config().set_user_option('email', 'Branch Identity <branch@identi.ty>')
+        b.get_config().set_user_option('email',
+                                       'Branch Identity <branch@identi.ty>')
         bzr_email = os.environ.get('BZREMAIL')
         if bzr_email is not None:
             del os.environ['BZREMAIL']
@@ -64,32 +65,48 @@
     def test_whoami_utf8(self):
         """verify that an identity can be in utf-8."""
         wt = self.make_branch_and_tree('.')
-        self.run_bzr('whoami', u'Branch Identity \u20ac <branch@identi.ty>', encoding='utf-8')
+        self.run_bzr('whoami', u'Branch Identity \u20ac <branch@identi.ty>',
+                     encoding='utf-8')
         bzr_email = os.environ.get('BZREMAIL')
         if bzr_email is not None:
             del os.environ['BZREMAIL']
         try:
             whoami = self.run_bzr("whoami", encoding='utf-8')[0]
-            self.assertEquals('Branch Identity \xe2\x82\xac <branch@identi.ty>\n', whoami)
-            whoami_email = self.run_bzr("whoami", "--email", encoding='utf-8')[0]
+            self.assertEquals('Branch Identity \xe2\x82\xac ' +
+                              '<branch@identi.ty>\n', whoami)
+            whoami_email = self.run_bzr("whoami", "--email",
+                                        encoding='utf-8')[0]
             self.assertEquals('branch@identi.ty\n', whoami_email)
         finally:
             if bzr_email is not None:
                 os.environ['BZREMAIL'] = bzr_email
 
     def test_whoami_ascii(self):
-        """verify that whoami doesn't totally break when in utf-8, using an ascii encoding."""
+        """
+        verify that whoami doesn't totally break when in utf-8, using an ascii
+        encoding.
+        """
         wt = self.make_branch_and_tree('.')
         b = bzrlib.branch.Branch.open('.')
-        b.get_config().set_user_option('email', u'Branch Identity \u20ac <branch@identi.ty>')
+        b.get_config().set_user_option('email', u'Branch Identity \u20ac ' +
+                                       '<branch@identi.ty>')
         bzr_email = os.environ.get('BZREMAIL')
         if bzr_email is not None:
             del os.environ['BZREMAIL']
         try:
             whoami = self.run_bzr("whoami", encoding='ascii')[0]
             self.assertEquals('Branch Identity ? <branch@identi.ty>\n', whoami)
-            whoami_email = self.run_bzr("whoami", "--email", encoding='ascii')[0]
+            whoami_email = self.run_bzr("whoami", "--email",
+                                        encoding='ascii')[0]
             self.assertEquals('branch@identi.ty\n', whoami_email)
         finally:
             if bzr_email is not None:
                 os.environ['BZREMAIL'] = bzr_email
+
+    def test_warning(self):
+        """verify that a warning is displayed if no email is given."""
+        self.make_branch_and_tree('.')
+        display = self.run_bzr('whoami', 'Branch Identity')[1]
+        self.assertEquals('"Branch Identity" does not seem to contain an '
+                          'email address.  This is allowed, but not '
+                          'recommended.\n', display)

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2006-07-10 23:50:51 +0000
+++ bzrlib/tests/test_bundle.py	2006-07-11 16:26:34 +0000
@@ -372,7 +372,7 @@
 
     def test_crlf_bundle(self):
         try:
-            read_bundle(StringIO('# Bazaar revision bundle v0.7\r\n'))
+            read_bundle(StringIO('# Bazaar revision bundle v0.8\r\n'))
         except BadBundle:
             # It is currently permitted for bundles with crlf line endings to
             # make read_bundle raise a BadBundle, but this should be fixed.
@@ -809,7 +809,8 @@
         wt.commit('add one', rev_id='a@cset-0-1')
         self.build_tree(['b1/two'])
         wt.add('two')
-        wt.commit('add two', rev_id='a@cset-0-2')
+        wt.commit('add two', rev_id='a@cset-0-2',
+                  revprops={'branch-nick':'test'})
 
         bundle_txt = StringIO()
         rev_ids = write_bundle(wt.branch.repository, 'a@cset-0-2',
@@ -820,7 +821,7 @@
 
     def check_valid(self, bundle):
         """Check that after whatever munging, the final object is valid."""
-        self.assertEqual(['a@cset-0-2'], 
+        self.assertEqual(['a@cset-0-2'],
             [r.revision_id for r in bundle.real_revisions])
 
     def test_extra_whitespace(self):
@@ -859,7 +860,26 @@
         # creates a blank line at the end, and fails if that
         # line is stripped
         self.assertEqual('\n\n', raw[-2:])
-        bundle_text = StringIO(raw[:-1])
-
-        bundle = read_bundle(bundle_txt)
-        self.check_valid(bundle)
+        bundle_txt = StringIO(raw[:-1])
+
+        bundle = read_bundle(bundle_txt)
+        self.check_valid(bundle)
+
+    def test_opening_text(self):
+        bundle_txt = self.build_test_bundle()
+
+        bundle_txt = StringIO("Some random\nemail comments\n"
+                              + bundle_txt.getvalue())
+
+        bundle = read_bundle(bundle_txt)
+        self.check_valid(bundle)
+
+    def test_trailing_text(self):
+        bundle_txt = self.build_test_bundle()
+
+        bundle_txt = StringIO(bundle_txt.getvalue() +
+                              "Some trailing\nrandom\ntext\n")
+
+        bundle = read_bundle(bundle_txt)
+        self.check_valid(bundle)
+

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2006-05-12 08:29:23 +0000
+++ bzrlib/tests/test_commit.py	2006-07-11 15:04:57 +0000
@@ -507,3 +507,16 @@
             ('deleted', 'filetoremove'),
             ],
             reporter.calls)
+
+    def test_commit_removals_respects_filespec(self):
+        """Commit respects the specified_files for removals."""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b'])
+        tree.add(['a', 'b'])
+        tree.commit('added a, b')
+        tree.remove(['a', 'b'])
+        Commit().commit(message='removed a', working_tree=tree, 
+                        specific_files='a')
+        basis = tree.basis_tree().inventory
+        self.assertIs(None, basis.path2id('a'))
+        self.assertFalse(basis.path2id('b') is None)

=== modified file 'bzrlib/tests/test_hashcache.py'
--- bzrlib/tests/test_hashcache.py	2006-03-31 00:43:21 +0000
+++ bzrlib/tests/test_hashcache.py	2006-07-12 05:22:48 +0000
@@ -1,4 +1,4 @@
-# (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,12 +16,13 @@
 
 import os
 import sha
+import stat
 import sys
 import time
 
 from bzrlib.errors import BzrError
 from bzrlib.hashcache import HashCache
-from bzrlib.tests import TestCaseInTempDir, TestSkipped
+from bzrlib.tests import TestCaseInTempDir, TestSkipped, TestCase
 
 
 def sha1(t):
@@ -29,117 +30,180 @@
 
 
 def pause():
-    if False:
-        return
-    if sys.platform in ('win32', 'cygwin'):
-        time.sleep(3)
-        return
-    # allow it to stabilize
-    start = int(time.time())
-    while int(time.time()) == start:
-        time.sleep(0.2)
-
-
-class FixThisError(Exception):
-    pass
-    
+    time.sleep(5.0)
+
 
 class TestHashCache(TestCaseInTempDir):
-
-    def test_hashcache(self):
-        """Functional tests for hashcache"""
-
+    """Test the hashcache against a real directory"""
+
+    def make_hashcache(self):
         # make a dummy bzr directory just to hold the cache
         os.mkdir('.bzr')
         hc = HashCache('.', '.bzr/stat-cache')
-
-        file('foo', 'wb').write('hello')
-        os.mkdir('subdir')
-        pause()
-
+        return hc
+
+    def reopen_hashcache(self):
+        hc = HashCache('.', '.bzr/stat-cache')
+        hc.read()
+        return hc
+
+    def test_hashcache_initial_miss(self):
+        """Get correct hash from an empty hashcache"""
+        hc = self.make_hashcache()
+        self.build_tree_contents([('foo', 'hello')])
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
         self.assertEquals(hc.miss_count, 1)
         self.assertEquals(hc.hit_count, 0)
 
-        # check we hit without re-reading
-        self.assertEquals(hc.get_sha1('foo'),
-                          'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        self.assertEquals(hc.miss_count, 1)
-        self.assertEquals(hc.hit_count, 1)
-
-        # check again without re-reading
-        self.assertEquals(hc.get_sha1('foo'),
-                          'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        self.assertEquals(hc.miss_count, 1)
-        self.assertEquals(hc.hit_count, 2)
-
-        # write new file and make sure it is seen
-        file('foo', 'wb').write('goodbye')
-        pause()
-        self.assertEquals(hc.get_sha1('foo'),
-                          '3c8ec4874488f6090a157b014ce3397ca8e06d4f')
-        self.assertEquals(hc.miss_count, 2)
-
-        # quickly write new file of same size and make sure it is seen
-        # this may rely on detection of timestamps that are too close
-        # together to be safe
-        file('foo', 'wb').write('g00dbye')
-        self.assertEquals(hc.get_sha1('foo'),
-                          sha1('g00dbye'))
-
-        file('foo2', 'wb').write('other file')
-        self.assertEquals(hc.get_sha1('foo2'), sha1('other file'))
-
-        os.remove('foo2')
-        self.assertEquals(hc.get_sha1('foo2'), None)
-
-        file('foo2', 'wb').write('new content')
-        self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-
+    def test_hashcache_new_file(self):
+        hc = self.make_hashcache()
+        self.build_tree_contents([('foo', 'goodbye')])
+        # now read without pausing; it may not be possible to cache it as its
+        # so new
+        self.assertEquals(hc.get_sha1('foo'), sha1('goodbye'))
+
+    def test_hashcache_nonexistent_file(self):
+        hc = self.make_hashcache()
+        self.assertEquals(hc.get_sha1('no-name-yet'), None)
+
+    def test_hashcache_replaced_file(self):
+        hc = self.make_hashcache()
+        self.build_tree_contents([('foo', 'goodbye')])
+        self.assertEquals(hc.get_sha1('foo'), sha1('goodbye'))
+        os.remove('foo')
+        self.assertEquals(hc.get_sha1('foo'), None)
+        self.build_tree_contents([('foo', 'new content')])
+        self.assertEquals(hc.get_sha1('foo'), sha1('new content'))
+
+    def test_hashcache_not_file(self):
+        hc = self.make_hashcache()
+        self.build_tree(['subdir/'])
         self.assertEquals(hc.get_sha1('subdir'), None)
 
-        # pause briefly to make sure they're not treated as new uncacheable
-        # files
+    def test_hashcache_load(self):
+        hc = self.make_hashcache()
+        self.build_tree_contents([('foo', 'contents')])
         pause()
-
-        self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-
-        # write out, read back in and check that we don't need to
-        # re-read any files
+        self.assertEquals(hc.get_sha1('foo'), sha1('contents'))
         hc.write()
-        del hc
-
-        hc = HashCache('.', '.bzr/stat-cache')
-        hc.read()
-
-        self.assertEquals(len(hc._cache), 2)
-        self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
+        hc = self.reopen_hashcache()
+        self.assertEquals(hc.get_sha1('foo'), sha1('contents'))
         self.assertEquals(hc.hit_count, 1)
-        self.assertEquals(hc.miss_count, 0)
-        self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
+
+    def test_hammer_hashcache(self):
+        hc = self.make_hashcache()
+        for i in xrange(10000):
+            self.log('start writing at %s', time.time())
+            f = file('foo', 'w')
+            try:
+                last_content = '%08x' % i
+                f.write(last_content)
+            finally:
+                f.close()
+            last_sha1 = sha1(last_content)
+            self.log("iteration %d: %r -> %r",
+                     i, last_content, last_sha1)
+            got_sha1 = hc.get_sha1('foo')
+            self.assertEquals(got_sha1, last_sha1)
+            hc.write()
+            hc = self.reopen_hashcache()
 
     def test_hashcache_raise(self):
         """check that hashcache can raise BzrError"""
-
-        os.mkdir('.bzr')
-        hc = HashCache('.', '.bzr/stat-cache')
-        ok = False
-
-        # make a best effort to create a weird kind of file
-        funcs = (getattr(os, 'mkfifo', None), getattr(os, 'mknod', None))
-        for func in funcs:
-            if func is None:
-                continue
-            try:
-                func('a')
-                ok = True
-                break
-            except FixThisError:
-                pass
-
-        if ok:
-            self.assertRaises(BzrError, hc.get_sha1, 'a')
+        hc = self.make_hashcache()
+        if not hasattr(os, 'mkfifo'):
+            raise TestSkipped('filesystem fifos not supported on this system')
+        os.mkfifo('a')
+        # It's possible that the system supports fifos but the filesystem
+        # can't.  In that case we should skip at this point.  But in fact
+        # such combinations don't usually occur for the filesystem where
+        # people test bzr.
+        self.assertRaises(BzrError, hc.get_sha1, 'a')
+
+
+class FakeHashCache(HashCache):
+    """Hashcache that consults a fake clock rather than the real one.
+
+    This lets us examine how old or new files would be handled, without
+    actually having to wait for time to pass.
+    """
+    def __init__(self):
+        # set root and cache file name to none to make sure we won't touch the
+        # real filesystem
+        HashCache.__init__(self, '.', 'hashcache')
+        self._files = {}
+        # simulated clock running forward as operations happen
+        self._clock = 0
+
+    def put_file(self, filename, file_contents):
+        abspath = './' + filename
+        self._files[abspath] = (file_contents, self._clock)
+
+    def _fingerprint(self, abspath):
+        entry = self._files[abspath]
+        return (len(entry[0]),
+                entry[1], entry[1],
+                10, 20,
+                stat.S_IFREG | 0600)
+
+    def _really_sha1_file(self, abspath):
+        if abspath in self._files:
+            return sha1(self._files[abspath][0])
         else:
-            raise TestSkipped('No weird file type could be created')
+            return None
+
+    def _cutoff_time(self):
+        return self._clock - 2
+
+    def pretend_to_sleep(self, secs):
+        self._clock += secs
+
+    
+class TestHashCacheFakeFilesystem(TestCaseInTempDir):
+    """Tests the hashcache using a simulated OS.
+    """
+
+    def make_hashcache(self):
+        return FakeHashCache()
+
+    def test_hashcache_miss_new_file(self):
+        """A new file gives the right sha1 but misses"""
+        hc = self.make_hashcache()
+        hc.put_file('foo', 'hello')
+        self.assertEquals(hc.get_sha1('foo'), sha1('hello'))
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 0)
+        # if we try again it's still too new; 
+        self.assertEquals(hc.get_sha1('foo'), sha1('hello'))
+        self.assertEquals(hc.miss_count, 2)
+        self.assertEquals(hc.hit_count, 0)
+
+    def test_hashcache_old_file(self):
+        """An old file gives the right sha1 and hits"""
+        hc = self.make_hashcache()
+        hc.put_file('foo', 'hello')
+        hc.pretend_to_sleep(20)
+        # file is new; should get the correct hash but miss
+        self.assertEquals(hc.get_sha1('foo'), sha1('hello'))
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 0)
+        # and can now be hit
+        self.assertEquals(hc.get_sha1('foo'), sha1('hello'))
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 1)
+        hc.pretend_to_sleep(3)
+        # and again
+        self.assertEquals(hc.get_sha1('foo'), sha1('hello'))
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 2)
+
+    def test_hashcache_invalidates(self):
+        hc = self.make_hashcache()
+        hc.put_file('foo', 'hello')
+        hc.pretend_to_sleep(20)
+        hc.get_sha1('foo')
+        hc.put_file('foo', 'h1llo')
+        self.assertEquals(hc.get_sha1('foo'), sha1('h1llo'))
+        self.assertEquals(hc.miss_count, 2)
+        self.assertEquals(hc.hit_count, 0)

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2006-06-20 03:30:14 +0000
+++ bzrlib/tests/test_options.py	2006-07-11 21:46:11 +0000
@@ -46,6 +46,10 @@
         out, err = self.run_bzr_captured(['help', '-r'], retcode=3)
         self.assertContainsRe(err, r'unknown option')
 
+    def test_allow_dash(self):
+        """Test that we can pass a plain '-' as an argument."""
+        self.assertEqual((['-'], {}), parse_args(cmd_commit(), ['-']))
+
 
 #     >>> parse_args('log -r 500'.split())
 #     (['log'], {'revision': [<RevisionSpec_int 500>]})

=== modified file 'bzrlib/tests/test_progress.py'
--- bzrlib/tests/test_progress.py	2006-06-21 03:14:59 +0000
+++ bzrlib/tests/test_progress.py	2006-07-10 23:00:44 +0000
@@ -14,8 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import os
 from StringIO import StringIO
 
+from bzrlib import errors
 from bzrlib.progress import (
         DummyProgress, ChildProgress,
         TTYProgressBar,
@@ -24,6 +26,7 @@
         )
 from bzrlib.tests import TestCase
 
+
 class FakeStack:
     def __init__(self, top):
         self.__top = top
@@ -44,6 +47,20 @@
             self.always_throttled = False
         
 
+class _TTYStringIO(StringIO):
+    """A helper class which makes a StringIO look like a terminal"""
+
+    def isatty(self):
+        return True
+
+
+class _NonTTYStringIO(StringIO):
+    """Helper that implements isatty() but returns False"""
+
+    def isatty(self):
+        return False
+
+
 class TestProgress(TestCase):
     def setUp(self):
         q = DummyProgress()
@@ -120,3 +137,174 @@
         # Updates after a second should not be squelched
         pb.update('me', 1, 1)
         self.assertFalse(pb.always_throttled)
+
+    def test_clear(self):
+        sio = StringIO()
+        pb = TTYProgressBar(to_file=sio, show_eta=False)
+        pb.width = 20 # Just make it easier to test
+        # This should not output anything
+        pb.clear()
+        # These two should not be displayed because
+        # of throttling
+        pb.update('foo', 1, 3)
+        pb.update('bar', 2, 3)
+        # So pb.clear() has nothing to do
+        pb.clear()
+
+        # Make sure the next update isn't throttled
+        pb.start_time -= 1
+        pb.update('baz', 3, 3)
+        pb.clear()
+
+        self.assertEqual('\r[=========] baz 3/3'
+                         '\r                   \r',
+                         sio.getvalue())
+
+    def test_no_eta(self):
+        # An old version of the progress bar would
+        # store every update if show_eta was false
+        # because the eta routine was where it was
+        # cleaned out
+        pb = InstrumentedProgress(to_file=StringIO(), show_eta=False)
+        # Just make sure this first few are throttled
+        pb.start_time += 5
+
+        # These messages are throttled, and don't contribute
+        for count in xrange(100):
+            pb.update('x', count, 300)
+        self.assertEqual(0, len(pb.last_updates))
+
+        # Unthrottle by time
+        pb.start_time -= 10
+
+        # These happen too fast, so only one gets through
+        for count in xrange(100):
+            pb.update('x', count+100, 200)
+        self.assertEqual(1, len(pb.last_updates))
+
+        pb.MIN_PAUSE = 0.0
+
+        # But all of these go through, don't let the
+        # last_update list grow without bound
+        for count in xrange(100):
+            pb.update('x', count+100, 200)
+
+        self.assertEqual(pb._max_last_updates, len(pb.last_updates))
+
+
+class TestProgressTypes(TestCase):
+    """Test that the right ProgressBar gets instantiated at the right time."""
+
+    def get_nested(self, outf, term, env_progress=None):
+        """Setup so that ProgressBar thinks we are in the supplied terminal."""
+        orig_term = os.environ.get('TERM')
+        orig_progress = os.environ.get('BZR_PROGRESS_BAR')
+        os.environ['TERM'] = term
+        if env_progress is not None:
+            os.environ['BZR_PROGRESS_BAR'] = env_progress
+        elif orig_progress is not None:
+            del os.environ['BZR_PROGRESS_BAR']
+
+        def reset():
+            if orig_term is None:
+                del os.environ['TERM']
+            else:
+                os.environ['TERM'] = orig_term
+            # We may have never created BZR_PROGRESS_BAR
+            # So we can't just delete like we can 'TERM' (which is always set)
+            if orig_progress is None:
+                if 'BZR_PROGRESS_BAR' in os.environ:
+                    del os.environ['BZR_PROGRESS_BAR']
+            else:
+                os.environ['BZR_PROGRESS_BAR'] = orig_progress
+
+        self.addCleanup(reset)
+
+        stack = ProgressBarStack(to_file=outf)
+        pb = stack.get_nested()
+        pb.start_time -= 1 # Make sure it is ready to write
+        pb.width = 20 # And it is of reasonable size
+        return pb
+
+    def test_tty_progress(self):
+        # Make sure the ProgressBarStack thinks it is
+        # writing out to a terminal, and thus uses a TTYProgressBar
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'xterm')
+        self.assertIsInstance(pb, TTYProgressBar)
+        try:
+            pb.update('foo', 1, 2)
+            pb.update('bar', 2, 2)
+        finally:
+            pb.finished()
+
+        self.assertEqual('\r/ [====   ] foo 1/2'
+                         '\r- [=======] bar 2/2'
+                         '\r                   \r',
+                         out.getvalue())
+
+    def test_dots_progress(self):
+        # Make sure the ProgressBarStack thinks it is
+        # not writing out to a terminal, and thus uses a 
+        # DotsProgressBar
+        out = _NonTTYStringIO()
+        pb = self.get_nested(out, 'xterm')
+        self.assertIsInstance(pb, DotsProgressBar)
+        try:
+            pb.update('foo', 1, 2)
+            pb.update('bar', 2, 2)
+        finally:
+            pb.finished()
+
+        self.assertEqual('foo: .'
+                         '\nbar: .'
+                         '\n',
+                         out.getvalue())
+
+    def test_no_isatty_progress(self):
+        # Make sure ProgressBarStack handles a plain StringIO()
+        import cStringIO
+        out = cStringIO.StringIO()
+        pb = self.get_nested(out, 'xterm')
+        pb.finished()
+        self.assertIsInstance(pb, DotsProgressBar)
+
+    def test_dumb_progress(self):
+        # Make sure the ProgressBarStack thinks it is writing out to a 
+        # terminal, but it is the emacs 'dumb' terminal, so it uses
+        # Dots
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'dumb')
+        pb.finished()
+        self.assertIsInstance(pb, DotsProgressBar)
+
+    def test_progress_env_tty(self):
+        # The environ variable BZR_PROGRESS_BAR controls what type of
+        # progress bar we will get, even if it wouldn't usually be that type
+        import cStringIO
+
+        # Usually, this would be a DotsProgressBar
+        out = cStringIO.StringIO()
+        pb = self.get_nested(out, 'dumb', 'tty')
+        pb.finished()
+        # Even though we are not a tty, the env_var will override
+        self.assertIsInstance(pb, TTYProgressBar)
+
+    def test_progress_env_dots(self):
+        # Even though we are in a tty, the env_var will override
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'xterm', 'dots')
+        pb.finished()
+        self.assertIsInstance(pb, DotsProgressBar)
+
+    def test_progress_env_none(self):
+        # Even though we are in a valid tty, no progress
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'xterm', 'none')
+        pb.finished()
+        self.assertIsInstance(pb, DummyProgress)
+
+    def test_progress_env_invalid(self):
+        out = _TTYStringIO()
+        self.assertRaises(errors.InvalidProgressBarType, self.get_nested,
+            out, 'xterm', 'nonexistant')

=== modified file 'bzrlib/tests/test_revisionnamespaces.py'
--- bzrlib/tests/test_revisionnamespaces.py	2006-05-24 00:06:33 +0000
+++ bzrlib/tests/test_revisionnamespaces.py	2006-07-08 20:38:36 +0000
@@ -27,6 +27,36 @@
 
 class TestRevisionNamespaces(TestCaseWithTransport):
 
+    def test_revno_n_path(self):
+        """Test revision specifiers.
+
+        These identify revisions by date, etc."""
+        wta = self.make_branch_and_tree('a')
+        ba = wta.branch
+        
+        wta.commit('Commit one', rev_id='a@r-0-1')
+        wta.commit('Commit two', rev_id='a@r-0-2')
+        wta.commit('Commit three', rev_id='a@r-0-3')
+
+        wtb = self.make_branch_and_tree('b')
+        bb = wtb.branch
+
+        wtb.commit('Commit one', rev_id='b@r-0-1')
+        wtb.commit('Commit two', rev_id='b@r-0-2')
+        wtb.commit('Commit three', rev_id='b@r-0-3')
+
+        self.assertEquals(RevisionSpec('revno:1:a/').in_history(ba),
+                          (1, 'a@r-0-1'))
+        # The argument of in_history should be ignored since it is
+        # redundant with the path in the spec.
+        self.assertEquals(RevisionSpec('revno:1:a/').in_history(None),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revno:1:a/').in_history(bb),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revno:2:b/').in_history(None),
+                          (2, 'b@r-0-2'))
+
+
     def test_revision_namespaces(self):
         """Test revision specifiers.
 

=== modified file 'bzrlib/tests/test_smart_add.py'
--- bzrlib/tests/test_smart_add.py	2006-07-11 18:34:09 +0000
+++ bzrlib/tests/test_smart_add.py	2006-07-12 14:24:06 +0000
@@ -191,6 +191,27 @@
         for path in paths:
             self.assertNotEqual(wt.path2id(path), None)
 
+    def test_add_multiple_dirs(self):
+        """Test smart adding multiple directories at once."""
+        added_paths = ['file1', 'file2',
+                       'dir1/', 'dir1/file3',
+                       'dir1/subdir2/', 'dir1/subdir2/file4',
+                       'dir2/', 'dir2/file5',
+                      ]
+        not_added = ['file6', 'dir3/', 'dir3/file7', 'dir3/file8']
+        self.build_tree(added_paths)
+        self.build_tree(not_added)
+
+        wt = self.make_branch_and_tree('.')
+        smart_add_tree(wt, ['file1', 'file2', 'dir1', 'dir2'])
+
+        for path in added_paths:
+            self.assertNotEqual(None, wt.path2id(path.rstrip('/')),
+                    'Failed to add path: %s' % (path,))
+        for path in not_added:
+            self.assertEqual(None, wt.path2id(path.rstrip('/')),
+                    'Accidentally added path: %s' % (path,))
+
 
 class TestAddNonNormalized(TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2006-06-21 02:33:40 +0000
+++ bzrlib/tests/test_ui.py	2006-07-12 02:45:22 +0000
@@ -26,6 +26,7 @@
 import bzrlib.errors as errors
 from bzrlib.progress import TTYProgressBar, ProgressBarStack
 from bzrlib.tests import TestCase
+from bzrlib.tests.test_progress import _TTYStringIO
 from bzrlib.ui import SilentUIFactory
 from bzrlib.ui.text import TextUIFactory
 
@@ -63,18 +64,41 @@
     def test_progress_note(self):
         stderr = StringIO()
         stdout = StringIO()
-        ui_factory = TextUIFactory()
-        pb = ui_factory.nested_progress_bar()
-        try:
-            pb.to_messages_file = stdout
-            ui_factory._progress_bar_stack.bottom().to_file = stderr
+        ui_factory = TextUIFactory(bar_type=TTYProgressBar)
+        pb = ui_factory.nested_progress_bar()
+        try:
+            pb.to_messages_file = stdout
+            ui_factory._progress_bar_stack.bottom().to_file = stderr
+            result = pb.note('t')
+            self.assertEqual(None, result)
+            self.assertEqual("t\n", stdout.getvalue())
+            # Since there was no update() call, there should be no clear() call
+            self.failIf(re.search(r'^\r {10,}\r$', stderr.getvalue()) is not None,
+                        'We cleared the stderr without anything to put there')
+        finally:
+            pb.finished()
+
+    def test_progress_note_clears(self):
+        stderr = StringIO()
+        stdout = StringIO()
+        # The PQM redirects the output to a file, so it
+        # defaults to creating a Dots progress bar. we
+        # need to force it to believe we are a TTY
+        ui_factory = TextUIFactory(bar_type=TTYProgressBar)
+        pb = ui_factory.nested_progress_bar()
+        try:
+            pb.to_messages_file = stdout
+            ui_factory._progress_bar_stack.bottom().to_file = stderr
+            # Create a progress update that isn't throttled
+            pb.start_time -= 10
+            pb.update('x', 1, 1)
             result = pb.note('t')
             self.assertEqual(None, result)
             self.assertEqual("t\n", stdout.getvalue())
             # the exact contents will depend on the terminal width and we don't
             # care about that right now - but you're probably running it on at
             # least a 10-character wide terminal :)
-            self.assertContainsRe(stderr.getvalue(), r'^\r {10,}\r$')
+            self.assertContainsRe(stderr.getvalue(), r'\r {10,}\r$')
         finally:
             pb.finished()
 
@@ -153,7 +177,7 @@
     def test_text_factory_prompts_and_clears(self):
         # a get_boolean call should clear the pb before prompting
         factory = bzrlib.ui.text.TextUIFactory()
-        factory.stdout = StringIO()
+        factory.stdout = _TTYStringIO()
         factory.stdin = StringIO("yada\ny\n")
         pb = self.apply_redirected(
             factory.stdin, factory.stdout, factory.stdout, factory.nested_progress_bar)
@@ -174,4 +198,4 @@
             "\r   *" 
             "\rwhat do you want\\? \\[y/n\\]:what do you want\\? \\[y/n\\]:", 
             output):
-            self.fail("didn't match factory output %r" % factory)
+            self.fail("didn't match factory output %r, %s" % (factory, output))

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-07-11 18:34:09 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-07-12 10:09:27 +0000
@@ -182,20 +182,6 @@
         self.assertEquals(list(tree.unknowns()),
                           ['hello.txt'])
 
-    def test_hashcache(self):
-        from bzrlib.tests.test_hashcache import pause
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['hello.txt',
-                         'hello.txt.~1~'])
-        tree.add('hello.txt')
-        pause()
-        sha = tree.get_file_sha1(tree.path2id('hello.txt'))
-        self.assertEqual(1, tree._hashcache.miss_count)
-        tree2 = WorkingTree.open('.')
-        sha2 = tree2.get_file_sha1(tree2.path2id('hello.txt'))
-        self.assertEqual(0, tree2._hashcache.miss_count)
-        self.assertEqual(1, tree2._hashcache.hit_count)
-
     def test_initialize(self):
         # initialize should create a working tree and branch in an existing dir
         t = self.make_branch_and_tree('.')

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2006-06-29 16:24:56 +0000
+++ bzrlib/trace.py	2006-07-12 02:49:48 +0000
@@ -256,6 +256,8 @@
 
 def report_exception(exc_info, err_file):
     exc_type, exc_object, exc_tb = exc_info
+    # Log the full traceback to ~/.bzr.log
+    log_exception_quietly()
     if (isinstance(exc_object, IOError)
         and getattr(exc_object, 'errno', None) == errno.EPIPE):
         print >>err_file, "bzr: broken pipe"

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2006-06-20 03:57:11 +0000
+++ bzrlib/uncommit.py	2006-07-11 21:09:40 +0000
@@ -43,6 +43,8 @@
         branch.lock_write()
         unlockable.append(branch)
 
+        pending_merges = []
+
         master = branch.get_master_branch()
         if master is not None:
             master.lock_write()
@@ -56,6 +58,12 @@
         files_to_remove = []
         for r in range(revno-1, len(rh)):
             rev_id = rh.pop()
+            rev = branch.repository.get_revision(rev_id)
+            # When we finish popping off the pending merges, we want
+            # them to stay in the order that they used to be.
+            # but we pop from the end, so reverse the order, and
+            # then get the order right at the end
+            pending_merges.extend(reversed(rev.parent_ids[1:]))
             if verbose:
                 print 'Removing revno %d: %s' % (len(rh)+1, rev_id)
 
@@ -68,6 +76,8 @@
             branch.set_revision_history(rh)
             if tree is not None:
                 tree.set_last_revision(branch.last_revision())
+                pending_merges.reverse()
+                tree.set_pending_merges(pending_merges)
     finally:
         for item in reversed(unlockable):
             item.unlock()

